'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var createDebug = _interopDefault(require('debug'));
var requireResolve = _interopDefault(require('resolve'));
var nodePath = _interopDefault(require('path'));

const defaultOptions = {
  config: {
    rootDir: null,
    srcDir: null
  },
  extensions: ['.mjs', '.js', '.vue', '.json']
};

/**
 * Assign default options
 *
 * @param {Object} defaultOptions
 * @param {Object} options
 *
 * @return {Object}
 */
const assignDefaultOptions = (defaultOptions, options) => Object.assign({}, defaultOptions, ...[...Object.entries(options)].map(([key, value]) => ({
  [key]: typeof value === 'object' ? Object.assign(defaultOptions[key], value) : value
})));

const debug = createDebug('eslint-plugin-import:resolver:nuxt-import');
const interfaceVersion = 2;
let firstCall = true;
function resolve(source, file, pluginOptions = {}) {
  if (requireResolve.isCore(source)) {
    return {
      found: true,
      path: null
    };
  }

  const options = assignDefaultOptions(defaultOptions, pluginOptions);

  if (firstCall) {
    firstCall = false;
    debug('Options', options);
  }

  debug('Source', source, 'File', file);
  let {
    rootDir,
    srcDir
  } = options.config;

  if (rootDir === null) {
    rootDir = process.cwd();
  } else if (!nodePath.isAbsolute(rootDir)) {
    rootDir = nodePath.join(process.cwd(), rootDir);
  }

  if (srcDir === null) {
    srcDir = rootDir;
  } else if (!nodePath.isAbsolute(srcDir)) {
    srcDir = nodePath.join(rootDir, srcDir);
  }

  let realSource = source;

  if (source.startsWith('~~') || source.startsWith('@@')) {
    realSource = nodePath.join(rootDir, source.substring(2));
  } else if (source.startsWith('~') || source.startsWith('@')) {
    realSource = nodePath.join(srcDir, source.substring(1));
  }

  try {
    const resolvedPath = requireResolve.sync(realSource, {
      extensions: options.extensions,
      basedir: nodePath.dirname(nodePath.resolve(file)),

      packageFilter(pkg) {
        const key = 'jsnext:main';

        if (pkg[key]) {
          pkg.main = key;
        }

        return pkg;
      }

    });
    debug('ResolvedPath', resolvedPath);
    return {
      found: true,
      path: resolvedPath
    };
  } catch (e) {
    debug('error', e);
    return {
      found: false
    };
  }
}

exports.interfaceVersion = interfaceVersion;
exports.resolve = resolve;
